package io.data2viz.timeFormat

import io.data2viz.time.date
import kotlin.test.Test

class LocaleTest : TestDate() {

    @Test
    fun timeFormat_specifier_defaults_to_en_US() {
        format("%c")(date(2000,4,1)) shouldBe "4/1/2000, 12:00:00 AM"
        format("%c")(date(2000,4,1, 23, 16, 4)) shouldBe "4/1/2000, 11:16:04 PM"
    }

    @Test
    fun timeFormat_specifier_for_locale_frFr() {
        val localeFr = Locale(Locales.fr_FR())
        localeFr.format("%c")(date(2000,1,1)) shouldBe "samedi, le  1 janvier 2000, 00:00:00"
    }

    @Test
    fun timeParse_specifier_defaults_to_en_US() {
        parse("%c")("4/1/2000, 12:00:00 AM") shouldBe date(2000,4,1)
        parse("%c")("4/1/2000, 11:16:04 PM") shouldBe date(2000,4,1, 23, 16, 4)
    }

    @Test
    fun timeParse_specifier_for_locale_frFr() {
        val localeFr = Locale(Locales.fr_FR())
        localeFr.parse("%c")("samedi, le  1 janvier 2000, 00:00:00") shouldBe date(2000,1,1)
        localeFr.parse("%c")("dimanche, le  9 janvier 2000, 20:4:12") shouldBe date(2000,1,9, 20, 4, 12)
        localeFr.parse("%c")("dimanche, le  9 janvier 2000, 20:04:12") shouldBe date(2000,1,9, 20, 4, 12)
    }
}
